/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

class ACFPlot extends Canvas {
    double acf[], chain[];
    int last, first, maxLag, numpoints;
    double mu, sigma2, varEst;
    
    Image sbuf = null;
    Dimension dold;
    
    ACFPlot() {
	reset(0,0, 1);
    }

    public void reset(int n, double theoMean, double theoVar) {
	setBackground(Color.white);
	setForeground(Color.black);

	acf = new double[n];
	chain = new double[n];
	maxLag = n;
	last = 0;
	first = 0;
	numpoints = 0;
	mu = theoMean;
	sigma2 = theoVar;
	varEst = 1;

	dold = new Dimension(0,0);
    }
    
    public void addPoint(double chainvalue) { // and update acf
	if( maxLag == 0 )
	    return;
	
	chain[last++] = chainvalue;
	numpoints++;
	last = (last < maxLag) ? last : 0;
	first = (first == last) ? first + 1 : first;
	first = (first < maxLag) ? first : 0;

	double t = chainvalue - mu;
	int i = last;
	varEst = 0;
	for( int j = 0; j < maxLag; j++ ) {
	    i = ( i > 0 ) ? i-1 : maxLag - 1; 
	    acf[j] += t * (chain[i] - mu);
	    varEst += acf[j];
	    if( i == first )
		break;	    
	}
	varEst = 2 * varEst - acf[0];
    }

    public void paint(Graphics g) {
	
	Dimension size = getSize();
	
	if( !size.equals(dold) ) { // new window size
	    sbuf = createImage(size.width, size.height);
	    dold.width = size.width;
	    dold.height = size.height;
	}

	Graphics screengc = g;
	g = sbuf.getGraphics();

	g.setColor(Color.white);
	g.fillRect(0,0,size.width, size.height);

	if( maxLag > 0 ) {
	    int w = (int)(size.width/maxLag);
	    int w2 = (int)(size.width/(2 * maxLag));
	    int h2 = (int)(size.height/2);
	    
	    g.setColor(Color.blue);
	    double v = acf[0];
	    for( int j = 0; j < maxLag; j++) {
		int y = (int)(size.height * acf[j] / (2 * v));
		int x = j * w + w2;
		g.drawLine( x, h2 - y, x, h2); 
//	    if( y >= 0 )
//		g.drawLine( j * size.width / maxLag, h2 - y, w, y);
//	    else
//		g.fillRect( j * size.width / maxLag, h2, w, -y);
	    }

	    g.setColor(Color.black);
	    for( int i = 0; i < maxLag; i += 20) 
		g.drawString( "" + i, i * w, 10);

	    g.drawString("Lag correlations estimated with " + numpoints + " iterations",
			 10, size.height - 15);
	    StringBuffer sbuf = new StringBuffer();
	    sbuf.append(v/varEst);
	    sbuf.setLength(7);
	    g.drawString("Efficiency : " + sbuf, 10, size.height - 5);
//	    g.drawString("Efficiency : " + (sigma2 * numpoints/varEst), 10, size.height - 15);
	}
	
	screengc.drawImage(sbuf, 0, 0, null);	
    }

    public void update(Graphics g) {
	paint(g);
    }    
}
