/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

class ChainTimeSeries extends Canvas implements MouseListener {
    double list[], mlist[][];
    int last, first, listSize;

    Image sbuf = null;
    Dimension dold;

    public MyObservable gopher;

    ChainTimeSeries(int n) {
	setBackground(Color.white);
	setForeground(Color.red);

	list = new double[n];
	mlist = new double[n][2];
	listSize = n;
	last = 0;
	first = 0;

	dold = new Dimension(0,0);

	gopher = new MyObservable();
	addMouseListener(this);
    }

    public void addPoint(double newValue) {

	newValue = (newValue < 0) ? 0 : newValue; // for protection
	newValue = (newValue > 1) ? 1 : newValue; 

	list[last++] = newValue;
	last = (last < listSize) ? last : 0;
	first = (first == last) ? first + 1 : first;
	first = (first < listSize) ? first : 0;

	mlist[last][0] = 0.0; // clear kinetic energy
	mlist[last][1] = 0.0; // clear potential energy
    }

    public void setEnergy(double k, double p) { // call this before addPoint
	mlist[last][0] = k;
	mlist[last][1] = p;
    }
    
    public void paint(Graphics g) {
	Dimension size = getSize();
	
	if( !size.equals(dold) ) { // new window size
	    sbuf = createImage(size.width, size.height);
	    dold.width = size.width;
	    dold.height = size.height;
	}

	Graphics screengc = g;
	g = sbuf.getGraphics();

	g.setColor(Color.white);
	g.fillRect(0,0,size.width, size.height);

      	int w = size.width;
	int w6 = size.width / 6;
	double h = (double)size.height/listSize;
	int h0 = (int)Math.floor(h) + 1;

	int x, y;
	int x0 = (int)(list[first] * w);
	int j = 0, y0 = 0;
	
	for( int i = first; ; i++ ) {
	    
	    i = (i < listSize) ? i : 0;
	    if( i == last )
		break;
	    
	    x = (int)(list[i] * w);
	    y = (int)( j * h);

	    g.setColor(Color.orange);
//	    g.drawLine((int)((0.5 - mlist[i]/3) * w), y0, (int)((0.5 + mlist[i]/3) * w), y);
	    int ikinetic = (int)(mlist[i][0] * w6);
	    int ipotential = (int)((mlist[i][0] + mlist[i][1]) * w6) - ikinetic;
	    g.fillRect(0, y0, ikinetic, h0);
	    g.setColor(Color.yellow);
	    g.fillRect(ikinetic, y0, ipotential, h0);
//	    g.fillRect(0, y0, (int)((0.5 + mlist[i]) * w), h);
	    g.setColor(Color.red);
	    g.drawLine(x0, y0, x, y);
	    
	    x0 = x;
	    y0 = y;
	    j++;
	}

//	g.setColor(Color.black);
//	g.drawRect(0,0,size.width-1, size.height-1);
	screengc.drawImage(sbuf, 0, 0, null);	
    }

    public void update(Graphics g) {
	paint(g);
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
 
    public void mousePressed(MouseEvent me) {
	me.consume();	
	Dimension d = getSize();

	double newChainValue = (double)me.getX() / d.width;
	gopher.forceChanged();
	gopher.notifyObservers(new Double(newChainValue));
    }

    public void mouseClicked(MouseEvent me) {
    }
    
    public void mouseEntered(MouseEvent me) {
    }
    public void mouseExited(MouseEvent me) {
    }
    public void mouseReleased(MouseEvent me) {
    }
}
