/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

class CoolingControlPanel extends Panel implements ActionListener, ItemListener {
    static final int LOG = 0;
    static final int POW = 1;
    static final int LIN = 2;
    static final int FRAC = 3;
    
    static final String LOGSTR = "Logarithmic Cooling";
    static final String POWSTR = "Power Cooling";
    static final String FRACSTR = "Fractional Power Cooling";
    static final String LINSTR = "Linear Cooling";
    
    static final double LOGDEFC = 1.0;
    static final double POWDEFC = 1.0;
    static final double POWDEFALPHA = 1.0;
    static final double FRACDEFC = 1.0;
    static final double FRACDEFALPHA = 1.0;
    static final double FRACDEFGAMMA = 1.0;
    static final double LINDEFC = 1.0;
    
    GridBagLayout gbl;
    GridBagConstraints gbc;

    Choice coolChoiceList;
    int coolCard;

    CardLayout coolCards;
    Panel coolCardPanel, logCard, powCard, linCard, fracCard;
    ParamInputField logParamC, powParamC, powParamAlpha, linParamC, fracParamC, fracParamAlpha,
	fracParamGamma;

    public MyObservable gopher;

    CoolingControlPanel() {
	setBackground(Color.white);
	setForeground(Color.black);

	gbl = new GridBagLayout();
	gbc = new GridBagConstraints();
	setLayout(gbl);
	
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.NONE;
	coolChoiceList = new Choice();
	gbl.setConstraints(coolChoiceList, gbc);
	add(coolChoiceList);
	coolChoiceList.addItemListener(this);

	gbc.gridy = GridBagConstraints.RELATIVE;
	gbc.gridx = 0;
	gbc.gridheight = GridBagConstraints.REMAINDER;
	coolCards = new CardLayout();
	coolCardPanel = new Panel(coolCards);
	gbl.setConstraints(coolCardPanel, gbc);
	add(coolCardPanel);
	
	coolChoiceList.add(LOGSTR);
	logCard = new Panel(new GridLayout(2,0));
	logCard.add(new Label(
	    "Temperature schedule T_n = C log(n)"));
	logParamC = new ParamInputField("C: ",
					 ParamInputField.POSITIVE, LOGDEFC,
					    this);
	logCard.add( logParamC );
	coolCardPanel.add(LOGSTR, logCard);

	coolChoiceList.add(POWSTR);
	powCard = new Panel(new GridLayout(4,0));
	powCard.add(new Label(
	    "Temperature schedule T_n = C alpha^n"));
	powParamC = new ParamInputField("C: ",
					 ParamInputField.POSITIVE, POWDEFC,
					    this);
	powParamAlpha = new ParamInputField("alpha: ",
					 ParamInputField.POSITIVE, POWDEFALPHA,
					    this);
	powCard.add( powParamC );
	powCard.add( powParamAlpha );
	coolCardPanel.add(POWSTR, powCard);

	coolChoiceList.add(FRACSTR);
	fracCard = new Panel(new GridLayout(4,0));
	fracCard.add(new Label(
	    "Temperature schedule T_n = C alpha^(n^gamma)"));
	fracParamC = new ParamInputField("C: ",
					 ParamInputField.POSITIVE, FRACDEFC,
					    this);
	fracParamAlpha = new ParamInputField("alpha: ",
					 ParamInputField.POSITIVE, FRACDEFALPHA,
					    this);
	fracParamGamma = new ParamInputField("gamma: ",
					 ParamInputField.POSITIVE, FRACDEFGAMMA,
					    this);
	fracCard.add( fracParamC );
	fracCard.add( fracParamAlpha );
	fracCard.add( fracParamGamma );
	coolCardPanel.add(FRACSTR, fracCard);

	coolChoiceList.add(LINSTR);
	linCard = new Panel(new GridLayout(2,0));
	linCard.add(new Label(
	    "Temperature schedule T_n = C * n"));
	linParamC = new ParamInputField("C: ",
					 ParamInputField.POSITIVE, LINDEFC,
					    this);
	linCard.add( linParamC );
	coolCardPanel.add(LOGSTR, linCard);

	gopher = new MyObservable();
    }

    public void doInit() {
	coolCard = LOG;
	coolChoiceList.select(LOGSTR);

	gopher.forceChanged();
	gopher.notifyObservers();
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
    
    public void itemStateChanged(ItemEvent ie) {
	String tmp = ie.getItem().toString();
	Vector gparams = new Vector(4,0);
	
	if( LOGSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(LOG));
	    gparams.addElement(new Double(logParamC.getValue()));
	    coolCards.show(coolCardPanel, LOGSTR);
	    coolCard = LOG;
	}
	else if( POWSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(POW));
	    gparams.addElement(new Double(powParamC.getValue()));
	    gparams.addElement(new Double(powParamAlpha.getValue()));
	    coolCards.show(coolCardPanel, POWSTR);
	    coolCard = POW;
	}
	else if( FRACSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(FRAC));
	    gparams.addElement(new Double(fracParamC.getValue()));
	    gparams.addElement(new Double(fracParamAlpha.getValue()));
	    gparams.addElement(new Double(fracParamGamma.getValue()));
	    coolCards.show(coolCardPanel, FRACSTR);
	    coolCard = FRAC;
	}
	else if( LINSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(LIN));
	    gparams.addElement(new Double(linParamC.getValue()));
	    coolCards.show(coolCardPanel, LINSTR);
	    coolCard = LIN;
	}	
	gopher.forceChanged();
	gopher.notifyObservers(gparams);
    }
    
    public void actionPerformed(ActionEvent ae) {
	Vector gparams = new Vector(4,0);
	
	switch(coolCard) {
	    case LOG:
		gparams.addElement(new Integer(LOG));
		gparams.addElement(new Double(logParamC.getValue()));
		break;
	    case LIN:
		gparams.addElement(new Integer(LIN));
		gparams.addElement(new Double(linParamC.getValue()));
		break;
	    case POW:
		gparams.addElement(new Integer(POW));
		gparams.addElement(new Double(powParamC.getValue()));
		gparams.addElement(new Double(powParamAlpha.getValue()));
		break;
	    case FRAC:
		gparams.addElement(new Integer(FRAC));
		gparams.addElement(new Double(fracParamC.getValue()));
		gparams.addElement(new Double(fracParamAlpha.getValue()));
		gparams.addElement(new Double(fracParamGamma.getValue()));
		break;
	    default:
	}
	gopher.forceChanged();
	gopher.notifyObservers(gparams);
    }
}








