/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.awt.image.*;

class CoupledFieldGraphPanel extends Panel {

    RandomField x = null;
    int step;
    int numbins;
    boolean collapseds[];

    Image sbuf = null;
    Dimension dold;

    RealInterval interval01;
   
    CoupledFieldGraphPanel() {
	setBackground(Color.white);
	setForeground(Color.black);
	step = -1;
	
	dold = new Dimension(0,0);
	interval01 = new RealInterval(0,1);
    }

    public void setField(RandomField r) {
	x = r;
	step++;
	if( (x != null) && ( x instanceof CoupledGaussianField) )
	    for( int k=0; k < numbins; k++ ) {
		if( !collapseds[k] )
		    collapseds[k] = ((CoupledGaussianField)x).hasCollapsed(k);
	    }
	repaint();
    }

    public void reset(RandomField r, int stepno) {
	x = r;
	step = stepno;
	if( (x != null) && (x instanceof CoupledGaussianField) ) {
	    numbins = ((CoupledGaussianField)x).numBins();
	    collapseds = new boolean[numbins];
	}
//	repaint();
    }

    public void paint(Graphics g) {
	Dimension d = getSize();
	
	if( !d.equals(dold) ) { // new window size
	    sbuf = createImage(d.width, d.height);
	    dold.width = d.width;
	    dold.height = d.height;
	}

	Graphics screengc = g;
	g = sbuf.getGraphics();

	g.setColor(Color.white);
	g.fillRect(0,0,d.width, d.height);
	
	if( x != null ) {
	    if( step < 1 ) {
		g.setColor(Color.red);
		for( int k = 0; k < numbins; k++ ) {
		    if( collapseds[k] )
			g.fillRect(d.width * k / numbins, 0, d.width / numbins, d.height);
		}
		if( x instanceof CoupledGaussianField ) {
		    g.setColor(Color.pink);
		    for( int l = 0; l < numbins; l++ ) 
			if( ((CoupledGaussianField)x).hasCollapsed(l) ) //&& (((CoupledGaussianField)x).numcollapsed[l] == 1) ) 
			    g.fillRect(d.width * l / numbins, 0, d.width / numbins, d.height);
		    g.setColor(Color.green);
		    double z0 = ((CoupledGaussianField)x).getz0();
		    double p = ((CoupledGaussianField)x).getProposal();
//		    g.drawLine((int)(d.width * z0), d.height, (int)(d.width * z0),
//			       (int)(d.height * ( 1 - p)) );
		    g.drawLine(0,(int)(d.height * (1 - p)), d.width, (int)(d.height * (1-p)));
		}
	    } else {
		g.setColor(Color.cyan);
		g.fillRect(0, 0, d.width, d.height);
	    }
	    g.setColor(Color.black);
	    g.drawString( "" + step, 10, 10);
	    x.displayGraph(g, d, interval01, interval01);
	}

	screengc.drawImage(sbuf, 0, 0, null);	
     }

    public void update(Graphics g) {
	paint(g);
    }
}














