/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class extends the basic random field on the interval [0,1].
 * It acts like a coupling machine, whereby an input random field
 * is modified at the sample path level, preserving the original
 * kernel density but replacing a section of the field with a single
 * common value. The common value is chosen independently, uniformly
 * on [0,1].
 * @author L.A. Breyer
 * @version 1.0
 */
class CoupledRandomField extends CouplableField {

    CouplableField oldField;
    double proposal, xi, z0;
    double propMean, propVariance2;

    CoupledRandomField(Random r, CouplableField field) {
	oldField = field;
	xi = r.nextDouble();
	z0 = r.nextDouble();
	proposal = field.newVariableAt(r, z0);
	propVariance2 = 2 * propVariance2;

	double top = z0;
	double bot = z0;
	double eps = 0.0;
    }

     public RealInterval omniCoupledRange() {
 	return new RealInterval(0,0);
     }
    
    public double valueAt(double x) {

	double fv = oldField.valueAt(x);
	double fvm = fv - propMean;
	double pm = proposal - propMean;
	
	if( oldField.kernelDensity(x, proposal) *
	    oldField.kernelDensity(z0, fv)
	    > xi * oldField.kernelDensity(x, fv) *
	    oldField.kernelDensity(z0, proposal) )
	    return proposal;
	else
	    return fv;
    }

    public double kernelDensity(double x, double y) {
	return oldField.kernelDensity(x,y);
    }

//    public double kernelMean(double x) {
//	return oldField.kernelMean(x);
//    }

//    public double kernelVariance(double x) {
//	return oldField.kernelVariance(x);
//    }

    public double newVariableAt(Random r, double x) {
	return oldField.newVariableAt(r, x);
    }
}
