/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.awt.image.*;

class FieldDensityGraph extends Canvas {

     int count;
    int numbins;
    double bins[][];
    Image img;
    Graphics ig;
    double max;

    boolean flag;
    
    Dimension size, remainder, binsize;
    
    FieldDensityGraph(int n) {
	reset(n);
	img = null;
	max = 0.0;
    }

    public void reset(int n) {
	bins = new double[n][n];
	numbins = n;
	img = null;
	max = 0.0;
    }

    public void setField(CouplableField x) {
	if( x == null )
	    return;

	max = 0.0;
	for( int i = 0; i < numbins; i++ ) 
	    for( int j= 0; j < numbins; j++ ) {
		double xv = x.kernelDensity(i/(double)numbins,
					    j/(double)numbins);
		max = (max < xv) ? xv : max;
		bins[i][j] = xv;
	    }
    }

    public void computeDensityGraph() {

	if( max <= 0.0 )
	    return;
	size = getSize();
	double bw = (double)size.width / numbins;
	double bh = (double)size.height / numbins;

	remainder = new Dimension( size.width % numbins,
				   size.height % numbins );
	    //	binsize = new Dimension( size.width / numbins, size.height / numbins);

	img = createImage(size.width - remainder.width,
			  size.height - remainder.height);
	ig = img.getGraphics();

 	for( int i = 0; i < numbins; i++ )
 	    for( int j = 0; j < numbins; j++ ) {
//		float c = (float)(Math.max( 0,1 + 0.01 * Math.log(bins[i][j]/max)));
//		float c = (float)(Math.pow(bins[i][j]/max, 0.9));
		float c = (float)(bins[i][j] / max);
		ig.setColor(new Color(Color.HSBtoRGB(c, c, c)));
		ig.fillRect((int)(i * bw), (int)((numbins - j) * bh),
			    (int)bw +1, (int)bh + 1);
	    }
    }

    public void paint(Graphics g) {
	if( img != null ) {
	    g.drawImage(img, 0, 0, null);
	}  else {
	    Dimension size = getSize();
	    g.setColor(Color.black);
	    g.fillRect(0,0,size.width, size.height);
	}

    }

    public void update(Graphics g) {
	paint(g);
    }
}







