/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This interface provides some data for the estimates used by the
 * Gaussian field coupler. It provides, for a given interval
 * (z0 - eps, z0 + eps), estimates on the law of X(z), assuming that
 * this random variable is distributed normally (or perhaps subnormally)
 */
interface GaussianEstimates {

    public double sigmaAt(double z0);
    public double muAt(double z0);
    public RealInterval sigmaRange(RealInterval r);
    public RealInterval muRange(RealInterval r);
    
	/**
	 * This method returns the maximal value of the standard deviation
	 * for the random variables X(z) where z0 - eps <= z <= z0 + eps
	 */
    public double maxSigma(double z0, double eps);
	/**
	 * This method returns the maximal value in modulus for the mean of
	 * random variables X(z) where z0 - eps <= z <= z0 + eps
	 */
    public double maxMu(double z0, double eps);
	/**
	 * This method returns the mean of X(z0)
	 */
    public double centerMu(double z0);
	/**
	 * Let q(z,dot) be the law of X(z). This method returns
	 * a bound of the form
	 * q(z,y)/q(z,x) <= b(x,y) for all z in the interval r.
	 */
    public double bEstimate(RealInterval r, double x, double y);
	/**
	 * Let q(z,dot) be the law of X(z). This method returns a bound
	 * q(z,y)/q(z,x) >= bTilde(x,y) for all z in the interval r.
	 */
    public double bTildeEstimate(RealInterval r, double x, double y);
	/**
	 * This returns the density q(x,y) of the law of X(x).
	 */
    public double kernelDensity(double x, double y);
}
