/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

class HybridMonteCarloField extends VectorRandomField {

    double pp, epsilon, epsilon2, xi, w, alpha, pSDev;
    double xhat[];
    Pdf target;
    int numLeaps;

	/**
	 * This constructor accepts a target density, and constructs
	 * a Hamiltonian Leapfrog scheme on top of an AR scheme on momenta,
	 * according to the prescription
	 * in Radford Neal's Techreport.
	 * @param r the random number generator
	 * @param a the coefficient for the autoregressive move
	 * @param epsilon the proper length of the Hamiltonian step
	 * @param pdf a Pdf used to compute the Hamiltonian step
	 */
    HybridMonteCarloField(Random r, double a,
			 double eps, int L, Pdf pdf) {
	alpha = a;
	pSDev = Math.sqrt(1 - a * a);
	xi = r.nextDouble();
	w = r.nextGaussian();
	target = pdf;
	epsilon = eps;
	numLeaps = L;
	if( r.nextDouble() > 0.5)
	    epsilon2 = eps/2;
	else
	    epsilon2 = -eps/2;
//	dvar2 = 2 * epsilon * epsilon * ( 1 - alpha * alpha );
    }

    HybridMonteCarloField respawn(Random r, double a,
			 double eps, int L, Pdf pdf) {
	alpha = a;
	pSDev = Math.sqrt(1 - a * a);
	xi = r.nextDouble();
	w = r.nextGaussian();
	target = pdf;
	epsilon = eps;
	numLeaps = L;
	if( r.nextDouble() > 0.5)
	    epsilon2 = eps/2;
	else
	    epsilon2 = -eps/2;
//	dvar2 = 2 * epsilon * epsilon * ( 1 - alpha * alpha );
	return this;
    }


    public int getDimension() {
	return 2;
    }
    
    public double[] valueAt(double x[]) {

	xhat = new double[2];

	double p = alpha * x[1] + pSDev * w;
	double xx = x[0];
	double pp = p;
	
	for( int k = 0; k < numLeaps; k++) {
	    pp = pp + epsilon2 * target.gradLogAt(xx);
	    xx = xx + 2 * epsilon2 * pp;
	    pp = pp + epsilon2 * target.gradLogAt(xx);
	}

	if( target.valueAt(xx) * Math.exp(- pp * pp / 2) > xi *
	    target.valueAt(x[0]) * Math.exp( - p * p / 2) ) {
	    xhat[0] = xx;
	    xhat[1] = pp;
	}
	else {
	    xhat[0] = x[0];
	    xhat[1] = p;
	}
	return xhat;
    }
}
