/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for using
 * as a proposal with the Independence sampler. The default
 * constructor creates a uniform random variable on [0,1].
 * @author L.A. Breyer
 * @version 1.0
 */
class IndependenceField extends CouplableField {

    double w;
    Pdf pdf;

    IndependenceField(Random r) {
	w = r.nextDouble();
	pdf = null;
    }

    IndependenceField respawn(Random r) {
	w = r.nextDouble();
	pdf = null;
	return this;
    }
    
    IndependenceField(Random r, Pdf p) {
	w = p.newVariable(r);
	pdf = p;
    }

    IndependenceField respawn(Random r, Pdf p) {
	w = p.newVariable(r);
	pdf = p;
	return this;
    }

    public double valueAt(double x) {
	return w;
    }

    public double kernelDensity(double x, double y) {
	if( pdf == null )
	    return 1;
	else
	    return pdf.valueAt(y);
    }

    public double kernelMean(double x) {
	if( pdf == null )
	    return 0.5;
	else
	    return pdf.getMean();
    }

    public double kernelVariance(double x) {
	if( pdf == null )
	    return 0.1;
	else
	    return pdf.getVariance();
    }

    public double newVariableAt(Random r, double x) {
	double w = pdf.newVariable(r);
	return w;
    }

    public RealInterval densityRatioRange(double z0, double epsilon) {
	return new RealInterval(1,1);
    }
}




