/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;

class PdfControlPanel extends Panel implements ActionListener, ItemListener {
    static final int NORMAL = 0;
    static final int EXPONENTIAL = 1;
    static final int USERDEFINED = 2;
    static final int SC = 3;
    static final int TRI = 4;
    static final int XCUBED = 5;

    static final String NORMSTR = "Gaussian density on [0,1]";
    static final String EXPSTR = "Exponential density on [0,1]";
    static final String UDSTR = "User defined density on [0,1]";
    static final String SCSTR = "SinCos density on [0,1]";
    static final String TRISTR = "Trimodal on [0,1]"; 
    static final String XCUBEDSTR = "x^3 on [0,1]";

    static final double NORMALDEFMU = 0.5;
    static final double NORMALDEFSIGMA2 = 0.002;
    static final double EXPDEFLAMBDA = 2.0;
    static final double SCDEFA = 1.0;
    static final double SCDEFB = 1.0;
    static final double TRIDEFSIGMA = 0.02;
    static final double TRIDEFA = 0.1; 
    static final double TRIDEFB = 0.6; 
    static final double TRIDEFC = 0.8; 

    public MyObservable gopher;
    

    Choice pdfChoiceList;
    int pdfCard;

    CardLayout pdfCards;
    Panel pdfCardPanel, normCard, expCard, userCard, scCard, triCard, xcubedCard;
    ParamInputField normParamMu, normParamSigma2, expParamLambda, scParamA, scParamB,
	triParamSigma, triParamA, triParamB, triParamC;
    Button userDrawButton, userAcceptButton, loadButton, saveButton;

    Pdf pdf;
    
    PdfControlPanel() {
	setBackground(Color.white);
	setForeground(Color.black);

	setLayout(new BorderLayout()); //gbl);
	
	pdfChoiceList = new Choice();
	add("North", pdfChoiceList);
	pdfChoiceList.addItemListener(this);
	
	pdfCards = new CardLayout();
	pdfCardPanel = new Panel(pdfCards);
	add("Center", pdfCardPanel);
	
	pdfChoiceList.add(NORMSTR);
	normCard = new Panel(new GridLayout(3,0));
	normCard.add(new Label("Truncated Gaussian density on [0,1]"));
	normParamMu = new ParamInputField("Mean: ",
					 ParamInputField.REAL, NORMALDEFMU,
					  this);
	normCard.add(normParamMu);
	normParamSigma2 = new ParamInputField("Variance",
					     ParamInputField.POSITIVE,
					      NORMALDEFSIGMA2,
					      this);
	normCard.add(normParamSigma2);
	pdfCardPanel.add(NORMSTR, normCard);

	
	pdfChoiceList.add(EXPSTR);
 	expCard = new Panel(new GridLayout(3,0));
 	expCard.add(new Label("Truncated Exponential density on [0,1]"));
 	expParamLambda = new ParamInputField("Lambda (1/Mean): ",
					     ParamInputField.POSITIVE,
					     EXPDEFLAMBDA, this);
 	expCard.add( expParamLambda ); 
	pdfCardPanel.add(EXPSTR, expCard);

	pdfChoiceList.add(UDSTR);
	userCard = new Panel(new GridLayout(2,0));
	userCard.add(new Label("Press buttons below to draw a density"));

	Panel kludge = new Panel(new FlowLayout(FlowLayout.CENTER, 20, 0));
	userDrawButton = new Button("New");
	userDrawButton.addActionListener(this);
	kludge.add(userDrawButton);
	userAcceptButton = new Button("Accept");
	userAcceptButton.addActionListener(this);
	kludge.add(userAcceptButton);
	loadButton = new Button("Load");
	loadButton.addActionListener(this);
	kludge.add(loadButton);
	saveButton = new Button("Save");
	saveButton.addActionListener(this);
	kludge.add(saveButton);
	userCard.add(kludge);
	pdfCardPanel.add(UDSTR, userCard);

	pdfChoiceList.add(SCSTR);
	scCard = new Panel(new GridLayout(2,0));
	Panel slp = new Panel();
	slp.add(new Label("1 + sin(a pi x) cos(b pi x)", Label.LEFT));
	slp.add(new Label(" on [0,1]"));
	scCard.add(slp);
	Panel spp = new Panel();
	scParamA = new ParamInputField("a: ",
					 ParamInputField.REAL, SCDEFA,
					  this);
	spp.add(scParamA);
	scParamB = new ParamInputField("b: ",
					     ParamInputField.REAL,
					      SCDEFB,
					      this);
	spp.add(scParamB);
	scCard.add(spp);
	pdfCardPanel.add(SCSTR, scCard);

	pdfCard = NORMAL;
	pdfChoiceList.select(NORMSTR);
	pdf = new Normal(NORMALDEFMU, NORMALDEFSIGMA2);	

	pdfChoiceList.add(TRISTR);
	triCard = new Panel(new GridLayout(2,0));
	Panel lp = new Panel();
	lp.add(new Label("e^[-(x-a)^2/2*sigma^2] ", Label.LEFT));
	lp.add(new Label("+ e^[-(x-b)^2/2*sigma^2] ", Label.LEFT));
	lp.add(new Label("+ e^[-(x-c)^2/2*sigma^2].", Label.LEFT));
	triCard.add( lp );

	triParamSigma = new ParamInputField("sigma: ",
					 ParamInputField.POSITIVE, TRIDEFSIGMA,
					  this);
	triParamA = new ParamInputField("a: ",
					 ParamInputField.REAL, TRIDEFA,
					  this);
	triParamB = new ParamInputField("b: ",
					 ParamInputField.REAL, TRIDEFB,
					  this);
	triParamC = new ParamInputField("c: ",
					 ParamInputField.REAL, TRIDEFC,
					  this);
	Panel pp = new Panel();
	pp.add(triParamA);
	pp.add(triParamB);
	pp.add(triParamC);
	pp.add(triParamSigma);
	triCard.add( pp);
	pdfCardPanel.add(TRISTR, triCard);

	pdfChoiceList.add(XCUBEDSTR);
	xcubedCard = new Panel(new GridLayout(2,0));
	Panel x3p = new Panel();
	x3p.add(new Label("x^3 truncated on [0,1]", Label.LEFT));
	xcubedCard.add( x3p );
	pdfCardPanel.add(XCUBEDSTR, xcubedCard);


	pdfCard = NORMAL;
	pdfChoiceList.select(NORMSTR);
	pdf = new Normal(NORMALDEFMU, NORMALDEFSIGMA2);	
	
	gopher = new MyObservable();
    }

    public void doInit() {
	pdfCard = NORMAL;
	pdfChoiceList.select(NORMSTR);
	pdfChoiceList.repaint();
	pdf = new Normal(NORMALDEFMU, NORMALDEFSIGMA2);
	gopher.forceChanged();
	gopher.notifyObservers(pdf);
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
    
    public void itemStateChanged(ItemEvent ie) {
	String tmp = ie.getItem().toString();

	if( NORMSTR.equals(tmp) ) {
	    pdfCard = NORMAL;
	    pdfCards.show(pdfCardPanel, NORMSTR);
	    pdf = new Normal(NORMALDEFMU, NORMALDEFSIGMA2);	   
	}
	else if( EXPSTR.equals(tmp) ) {
	    pdfCard = EXPONENTIAL;
	    pdfCards.show(pdfCardPanel, EXPSTR);
	    pdf = new Exponential(EXPDEFLAMBDA);
	}
	else if( UDSTR.equals(tmp) ) {
	    pdfCard = USERDEFINED;
	    pdfCards.show(pdfCardPanel, UDSTR);
	    pdf = null;
	}
	else if( SCSTR.equals(tmp) ) {
	    pdfCard = SC;
	    pdfCards.show(pdfCardPanel, SCSTR);
	    pdf = new SinCos(SCDEFA,SCDEFB);
	}
	else if( TRISTR.equals(tmp) ) {
	    pdfCard = TRI;
	    pdfCards.show(pdfCardPanel, TRISTR);
	    pdf = new Trimodal(TRIDEFSIGMA,TRIDEFA,TRIDEFB,TRIDEFC);
	}
	else if( XCUBEDSTR.equals(tmp) ) {
	    pdfCard = XCUBED;
	    pdfCards.show(pdfCardPanel, XCUBEDSTR);
	    pdf = new Xcubed();
	}
	gopher.forceChanged();
	gopher.notifyObservers(pdf);
    }
    
    public void actionPerformed(ActionEvent ae) {
	gopher.forceChanged();
	switch(pdfCard) {
	case NORMAL:
	    pdf = new Normal(normParamMu.getValue(),
			     normParamSigma2.getValue());
	    gopher.notifyObservers(pdf);
	    break;
	case EXPONENTIAL:
	    pdf = new Exponential(expParamLambda.getValue());
	    gopher.notifyObservers(pdf);
	    break;
	case SC:
	    pdf = new SinCos(scParamA.getValue(), scParamB.getValue());
	    gopher.notifyObservers(pdf);
	    break;
	case TRI:
	    pdf = new Trimodal(triParamSigma.getValue(), triParamA.getValue(),
			       triParamB.getValue(), triParamC.getValue());
	    gopher.notifyObservers(pdf);
	    break;
	case XCUBED:
	    pdf = new Xcubed();
	    gopher.notifyObservers(pdf);
	    break;
	case USERDEFINED:
	    String str = ae.getActionCommand();
	    gopher.notifyObservers(str);
	}
    }
}









































































