/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a real valued random field on the interval (0,1).
 * It can and should be thought of as a function with random values. The
 * density for the value at the point x is given by a kernel density.
 * @author L.A. Breyer
 * @version 1.0
 */
abstract class RandomField extends RealFunction {

	/**
	 * This method has the same effect as the method valueAt, except
	 * that the random numbers used are different. Use this when you
	 * need to generate a large number of IID variables distributed
	 * as X(x). This method must be overridden, as it returns 0 by default.
	 * @param r the random number generator
	 * @param x the location
	 * @return a random variable distributed in the same way as X(x)
	 * @see RealFunction
	 */
    public double newVariableAt(Random r, double x) {
	return 0;
    }

	// returns pi raised to a power beta. Should be overridden
    public double hotValueAt(double x, double beta) {
	return Math.pow( valueAt(x), beta );
    }
}




