/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for using
 * as a proposal with the Random walk Metropolis sampler. The default
 * constructor creates a uniform[-1,1] increment.
 * @author L.A. Breyer
 * @version 1.0
 */
class RandomWalkField extends CouplableField {

    double w;
    double sigma;
    Pdf pdf;

    RandomWalkField(Random r) {
	w = r.nextDouble() * 2 - 1;
	pdf = null;
    }

	/**
	 * This constructor accepts a proposal density and a value of
	 * s, the magnification factor for the increment.
	 * @param r the random number generator
	 * @param p the density for the increment
	 * @param s the standard deviation for the increment
	 */
    RandomWalkField(Random r, Pdf p, double s) {
	w = s * p.newVariable(r);
	sigma = s;
	pdf = p;
    }

    public double valueAt(double x) {
	return x + w;
    }

    public double kernelDensity(double x, double y) {
//	if( pdf == null )
//	    return ( (y - x > 1) || (y-x < -1) ? 0 : 1 );
//	else
	return pdf.valueAt(y - x);
    }

     public double kernelMean(double x) {
//	if( pdf == null )
//	    return x;
//	else
	 return x + pdf.getMean();
    }

    public double kernelVariance(double x) {
//	if( pdf == null )
//	    return 4/3;
//	else
	return sigma * sigma * pdf.getVariance();
    }   

    public double newVariableAt(Random r, double x) {
	double w = sigma * pdf.newVariable(r);
	return w;
    }

	/**
	 * this is a dummy method.
	 */
    public RealInterval densityRatioRange(double z0, double epsilon) {
	return new RealInterval(1,1);
    }
}
