/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.awt.image.*;

class RoadMapControlPanel extends Panel implements ActionListener, ItemListener {

    final static double DEFHISTDELAY = 5;
    final static double DEFITDELAY = 0;
    final static double DEFNUMBINS = 50;
    
    final static String RESETSTR = "Reset hist.";

    final static String CB1STR = "view proposal";
    final static String CB2STR = "view metropolized";
    final static String CB3STR = "no transition graph";
    
    final static String ITDELAY = "delay";
    final static String ITBTHIST = "hist. updates";
    final static String NUMBINS = "bins";
    
    public MyObservable gopher;
    Button resetButton;
    ParamInputField delayParam;
    ParamInputField histDelayParam;
    ParamInputField numbinsParam;

    CheckboxGroup cbg;
    Checkbox cb1, cb2, cb3; 
    
    RoadMapControlPanel() {
	setBackground(Color.white);
	setForeground(Color.black);
	setLayout(new GridLayout(3,0));

	Panel p1 = new Panel(new GridLayout(2, 2));
	
	resetButton = new Button(RESETSTR);
	resetButton.addActionListener(this);
	p1.add(resetButton);

	histDelayParam = new ParamInputField(ITBTHIST,
					     ParamInputField.POSITIVE, DEFHISTDELAY,
					     this);
	//	p1.add(histDelayParam);
	p1.add(new Panel());

	delayParam = new ParamInputField(ITDELAY,
					 ParamInputField.POSITIVE, DEFITDELAY, this);
	p1.add(delayParam);

	numbinsParam = new ParamInputField(NUMBINS,
					   ParamInputField.POSITIVE, DEFNUMBINS, this);
	p1.add(numbinsParam);

	add(p1);

	Panel p2 = new Panel(new GridLayout(2,0));
	cbg = new CheckboxGroup();
	cb1 = new Checkbox(CB1STR, cbg, false);
	cb1.addItemListener(this);
	p2.add(cb1);
	cb2 = new Checkbox(CB2STR, cbg, true);
	cb2.addItemListener(this);
	p2.add(cb2);

	add(p2);

	cb3 = new Checkbox(CB3STR, null, false);	
	cb3.addItemListener(this);
	add(cb3);
	
	gopher = new MyObservable();
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
    
    public void actionPerformed(ActionEvent ae) {
	String str = ae.getActionCommand();	
	if( str.equals(RESETSTR) ) {
	    gopher.forceChanged();
	    gopher.notifyObservers(str);
	} else {
	    gopher.forceChanged();
	    gopher.notifyObservers(ae);
	}
    }

    public void itemStateChanged(ItemEvent ie) {
	if( (ie.getItemSelectable() == cb1) || (ie.getItemSelectable() == cb2) ) {
	    gopher.forceChanged();
	    gopher.notifyObservers(cbg.getSelectedCheckbox().getLabel());
	} else if( ie.getItemSelectable() == cb3 ) {
	    gopher.forceChanged();
	    if(cb3.getState() == false)
		gopher.notifyObservers("graphon");
	    else
		gopher.notifyObservers("graphoff");
	}
//	cb3.getLabel());
    }
}
