/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for using
 * as a proposal with the Random walk Metropolis sampler, using a Student t
 * proposal distribution.
 * @author L.A. Breyer
 * @version 1.0
 */
class StudentRandomWalkField extends CouplableField {
// implements GaussianEstimates {

    double w;
    double sigma, oosigmanu, nuexp;
    int nu;

	/**
	 * This constructor accepts a value of
	 * s, the magnification factor for the increment.
	 * @param r the random number generator
	 * @param s the standard deviation for the increment
	 */
    StudentRandomWalkField(MyRandom r, double s, int n) {
	nu = n;
	sigma = s;
	w = sigma * r.nextStudent(n);
	oosigmanu = (double)1/(sigma * nu);
	nuexp = - (nu + 1)/2;
    }

    StudentRandomWalkField respawn(MyRandom r, double s, int n) {
	nu = n;
	sigma = s;
	w = sigma * r.nextStudent(n);
	oosigmanu = (double)1/(sigma * nu);
	nuexp = - (nu + 1)/2;
	return this;
    }

    public double valueAt(double x) {
	return x + w;
    }

    public double kernelDensity(double x, double y) {
	double t = (y-x);
	return Math.pow( 1 + x * x * oosigmanu, nuexp );
    }

    public double maxSigma(double z0, double eps) {
	return sigma;
    }

    public double maxMu(double z0, double eps) {
	return eps;
    }

    public double centerMu(double z0) {
	return z0;
    }    

// 	/**
// 	 * note: estimate assumes that r is a subinterval of [0,1]
// 	 */
//     public double bEstimate(RealInterval r, double x, double y) {
// 	double t = y * y - x * x;
// 	if( y > x )
// 	    t -= 2 * r.end * (y - x);
// 	else
// 	    t -= 2 * r.beginning * (y - x);

// 	return Math.exp( - t / (2 * sigma2) );
//     }

// 	/**
// 	 * note: estimate assumes that r is a subinterval of [0,1]. 
// 	 */
//     public double bTildeEstimate(RealInterval r, double x, double y) {
// 	double t = y * y - x * x;
// 	if( y > x )
// 	    t -= 2 * r.beginning * (y - x);
// 	else
// 	    t -= 2 * r.end * (y - x);

// 	return Math.exp( - t / (2 * sigma2) );
//     }
    
//    public double kernelMean(double x) {
//	return x;
//   }

//    public double kernelVariance(double x) {
//	return sigma2;
//    }

    public double newVariableAt(MyRandom r, double x) {
	double w = r.nextStudent(nu);
	return x + sigma * w;
    }
}
