/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class extends the basic Hastings-Metropolis machine, by
 * optimizing the valueAt function for Random Walk type proposal fields.
 * @author L.A. Breyer
 * @version 1.0
 */
class SymPropHMField extends HMField {

    SymPropHMField(Random r, Pdf f, CouplableField p) {
	super(r,f,p);
    }

    SymPropHMField respawn2(Random r, Pdf f, CouplableField p) {
	target = f;
	proposal = p;
	xi = r.nextDouble();
	return this;
    }
	/**
	 * This method returns the value at x of the Hastings Metropolis
	 * random field z goesto X(z). The proposal kernel q(z,y) is assumed
	 * to be symmetric. 
	 * @param x
	 * @return X(x)
	 */
    public double valueAt(double x) {
	double y = proposal.valueAt(x);
	if( target.valueAt(y) >=
	    xi * target.valueAt(x) )
	    return y;
	else
	    return x;
    }

	// hot value requires inverse temperature beta
    public double hotValueAt(double x, double beta) {
	double y = proposal.valueAt(x);
	if( target.valueAt(x) == 0 )
	    return y;
	else if( Math.pow( target.valueAt(y) / target.valueAt(x), beta ) > xi )
	    return y;
	else
	    return x;
    }
    
    public double newVariableAt(Random r, double x) {
	double xi = r.nextDouble();
	double y = proposal.valueAt(x);
	if( target.valueAt(y) >
	    xi * target.valueAt(x) )
	    return y;
	else
	    return x;	
    }
}
