/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.awt.image.*;

class TimeHistogram extends Canvas {

    int count, numbins, max;
    int bins[];
    Dimension dold;
    Image sbuf = null;

    TimeHistogram(int n) {
	setFont(new Font("Dialog",Font.PLAIN, 8));
	reset(n);
	dold = new Dimension(0,0);
    }

    public void reset(int n) {
	bins = new int[n];
	numbins = n;
	count = 0;
	max = 1;
    }

    public void addTime(int x) {
	count++;
	if( x < numbins ) {
	    bins[x]++;
	    max = Math.max(max, bins[x]);
	} else {
	    bins[numbins - 1]++;
	    max = Math.max(max, bins[numbins - 1]);
	}	
    }

    public void paint(Graphics g) {
	if((count == 0) || (numbins == 0))
	    return;
	
	Dimension size = getSize();
	if( !size.equals(dold) ) { // new window size
	    sbuf = createImage(size.width, size.height);
	    dold.width = size.width;
	    dold.height = size.height;
	}

	Graphics screengc = g;
	
	g = sbuf.getGraphics();

	g.setColor(Color.white);
	g.fillRect(0,0,size.width, size.height);
	g.setColor(Color.black);

	double w = (double)(size.width) / numbins;
	int xoffset = (int)Math.floor(w/2);
	int h = size.height / 2;
	
	for(int i = 0; i < numbins; i++) {
	    int y = (int)(bins[i] * h / max);
	    int x = (int)(i * w);
	    int wx = (int)Math.floor(((i+1) * w) - x) + 1;
	    g.fillRect(x, size.height - y, wx, y);
//	    g.drawLine(xoffset + x, size.height - y, xoffset + x, size.height);
	}

	for( int k = 0; k < numbins; k += 100 )
	    g.drawString("" + k, k * size.width / numbins, 10);

	g.drawString( "Count: " + count, 10, 30);

	screengc.drawImage(sbuf, 0, 0, null);
    }

    public void update(Graphics g) {
	paint(g);
    }
}




